package SlackBuild::URI;
use strict;
use warnings;
use URI;
use Carp;
use LWP::UserAgent;
use File::Basename;
use File::Spec;
# use LWP::Protocol::https;
# use LWP::Protocol::file;
# use LWP::Protocol::ftp;

our $AUTOLOAD;

sub AUTOLOAD {
    my $self = shift;

    # The Perl interpreter places the name of the
    # message in a variable called $AUTOLOAD.

    # DESTROY messages should never be propagated.
    return if $AUTOLOAD =~ /::DESTROY$/;

    # Remove the package name.
    $AUTOLOAD =~ s/^.*:://;
    
    # Pass the message to the delegate.
    return $self->{_uri}->$AUTOLOAD(@_);
}

sub _valid_scheme {
    my $scheme = shift->scheme;
    return eval { require "LWP/Protocol/$scheme.pm" }; 
}

sub new {
    my $class = shift;
    my $uri = new URI(@_);
    $uri->scheme('file') unless $uri->scheme;
    my $self = bless { _uri => $uri }, $class;
    croak "$self: unsupported scheme"
    	unless $self->_valid_scheme;
    return $self;
}

sub clone {
    my $self = shift;
    return new SlackBuild::URI($self);
}

sub new_abs {
    my ($self, $arg) = @_;
    my $s = $self->clone;
    $s->path(File::Spec->catfile($self->path, $arg));
    return $s;
}

use overload '""' => sub { shift->as_string };

sub download {
    my $self = shift;
    my $dst = shift || basename($self->path);
    my $scheme = $self->scheme;
    require "LWP/Protocol/$scheme.pm";
    print "downloading $self\n";
    my $ua = LWP::UserAgent->new();
    my $response = $ua->get($self->as_string, ':content_file' => $dst);
    $self->download_response($response);
    return $response->is_success;
}

sub download_response {
    my $self = shift;
    croak "too many arguments" if (@_ > 1);
    if (my $v = shift) {
	$self->{_response} = $v;
    }
    return $self->{_response};
}

sub download_success {
    my $self = shift;
    return $self->download_response->is_success;
}

sub download_status {
    my $self = shift;
    return $self->download_response->status_line;
}

sub downloaded_html {
    my $self = shift;
    return $self->download_response->content_type =~ m{^(?:application/xhtml\+xml|text/(?:css|html))$};
}

1;
