package SlackBuild::match;
use strict;
use warnings;
use parent 'Exporter';
use Carp;
use Scalar::Util qw(looks_like_number);

our @EXPORT = qw(match);

my %matchtab = (
    -eq => sub {
	my ($a, $b) = @_;
	(looks_like_number($a) && looks_like_number($b)) ? $a == $b : $a eq $b;
    },
    '=' => '-eq',
    '==' => '-eq',
    
    -ne => sub {
	my ($a, $b) = @_;
	(looks_like_number($a) && looks_like_number($b)) ? $a != $b : $a ne $b;
    },
    '!=' => '-ne',
    
    -lt => sub {
	my ($a, $b) = @_;
	(looks_like_number($a) && looks_like_number($b)) ? $a < $b : $a lt $b;
    },
    '<' => '-lt',
    
    -le => sub {
	my ($a, $b) = @_;
	(looks_like_number($a) && looks_like_number($b)) ? $a <= $b : $a le $b;
    },
    '<=' => '-le',
    
    -gt => sub {
	my ($a, $b) = @_;
	(looks_like_number($a) && looks_like_number($b)) ? $a > $b : $a gt $b;
    },
    '>' => '-gt',
    
    -ge => sub {
	my ($a, $b) = @_;
	(looks_like_number($a) && looks_like_number($b)) ? $a > $b : $a ge $b;
    },
    '>=' => '-ge',
    
    -in => sub {
	my ($a, $b) = @_;
	croak 'argument to -in must be array'
	    unless ref($b) eq 'ARRAY';
	foreach my $v (@$b) {
	    return 1
		if (looks_like_number($a) && looks_like_number($v))
		    ? $a == $v : $a eq $v;
	}
    }
);

sub match {
    my ($val, $pred) = @_;

    my ($fun, $arg);
    if (ref($pred) eq 'ARRAY') {
	croak "array must have 2 elements" unless @$pred == 2;
        ($fun, $arg) = @$pred;
    } elsif (ref($pred) eq 'HASH') {
	croak "array must have 1 key" unless keys(%$pred) == 1;
	($fun, $arg) = each %$pred;
    } elsif ($pred =~ m{^([<=>]=?)(.*)}) {
	$fun = $1;
	$arg = $2;
    } else {
	$fun = '-eq';
	$arg = $pred;
    }

    while (exists($matchtab{$fun}) && !ref($matchtab{$fun})) {
	$fun = $matchtab{$fun};
    }
    
    croak "unknown predicate: $fun"
	unless (exists($matchtab{$fun}));

    &{$matchtab{$fun}}($val, $arg);
}
